/**
 * @file dmxcue.h
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Represents a DMX data cue in a queue
**/
#ifndef DMXCUE_H
#define DMXCUE_H
#include <stdio.h>


/* Trigger type definitions */
#define DMXCUE_TRIGGER_DEFAULT   0
#define DMXCUE_TRIGGER_ONSTEP    0
#define DMXCUE_TRIGGER_ONTIME    1
#define DMXCUE_TRIGGER_ONSIGNAL  2


/**
 * Represents a channel->value combination
**/
typedef struct dmxpair {
	int channel;
	unsigned char value;
	/* Tree construction data */
	struct dmxpair *next;
} dmxpair_t;

/**
 * Search Tree of dmxpairs
**/
typedef struct dmxlist {
	/* Tree root node */
	dmxpair_t *first;
	/* Size / Number of elements */
	int size;
	/* trigger values */
	int trigger_type;
	int trigger_time;
	int trigger_fade;
} dmxlist_t;


/* Initiate/Reset/Destroy a list */
dmxlist_t *dmxcue_init();
int dmxcue_clear(dmxlist_t*);
void dmxcue_destroy(dmxlist_t*);
/* Adds an element to the tree */
int dmxcue_set(int, unsigned char, dmxlist_t*);
/* Adds an element to the tree if not existing or adds the value to existing */
int dmxcue_setadd(int, int, dmxlist_t*);
/* Removes an object from the list */
int dmxcue_unset(int, dmxlist_t*);
/* Get the next element from a list */
dmxpair_t *dmxcue_get_next(dmxpair_t*);
/* Get a special channels object */
dmxpair_t *dmxcue_get_channel_obj(int, dmxlist_t*);
/* Gets a channels value */
unsigned char dmxcue_get_channel_value(int, dmxlist_t*);
/* saves into a file */
long dmxcue_save(FILE*, dmxlist_t*);
/* loads from a file */
long dmxcue_load(FILE*, dmxlist_t*);
/* Checks if a cue ist empty */
int dmxcue_is_empty(dmxlist_t*);

#endif /* DMXCUE_H */
